#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Examp Petzval Lens"""


import numpy as np
import pkg_resources
""" Looking for if KrakenOS is installed, if not, it assumes that
an folder downloaded from github is run"""

required = {'KrakenOS'}
installed = {pkg.key for pkg in pkg_resources.working_set}
missing = required - installed

if missing:
    print("Not installed")
    import sys
    sys.path.append("../..")


import KrakenOS as Kos




OBJ=Kos.surf()
OBJ.Rc=0.0
OBJ.Thickness=10.0
OBJ.Glass="AIR"
OBJ.Diameter=56


STO=Kos.surf()
STO.Rc=0.0
STO.Thickness=10.0
STO.Glass="AIR"
STO.Diameter=56


s2=Kos.surf()
s2.Rc=91.66860362426125
s2.Thickness=2.16760007E+1
s2.Glass="BALKN3"
s2.Diameter=56
s2.DespX = 1.0
s2.AxisMove = 0

s3=Kos.surf()
s3.Rc=-60.18807811401379
s3.Thickness=3.48399646
s3.Glass="F4"
s3.Diameter=56
s3.DespY = 2.0
s3.AxisMove = 0

s4=Kos.surf()
s4.Rc=3972.618614963046
s4.Thickness=7.65093542E+1
s4.Glass="AIR"
s4.Diameter=56
s4.TiltX = 3.0
s4.AxisMove = 0

s5=Kos.surf()
s5.Rc=35.206739696593594
s5.Thickness=1.7360596E+1
s5.Glass="N-BK7"
s5.Diameter=48
s5.TiltY = 4.0
s5.AxisMove = 0

s6=Kos.surf()
s6.Rc=-58.825151255787866
s6.Thickness=3.48399646
s6.Glass="F2"
s6.Diameter=48
s6.DespX = -1
s6.AxisMove = 0

s7=Kos.surf()
s7.Rc=-166.14088338204223
s7.Thickness=1.93995258E+1
s7.Glass="AIR"
s7.Diameter=48
s7.DespY = -2
s7.AxisMove = 0


s8=Kos.surf()
s8.Rc=-27.78333066229023
s8.Thickness=1.96964573
s8.Glass="F2"
s8.Diameter = 32
s8.TiltX = -3
s8.AxisMove = 0


s9=Kos.surf()
s9.Rc=1111723.0516534068
s9.Thickness=15.1962995812
s9.Glass="AIR"
s9.Diameter=32

IMA=Kos.surf()
IMA.Rc=0.0
IMA.Thickness=0.0
IMA.Glass="AIR"
IMA.Diameter=13.98677

A = [OBJ, STO, s2, s3, s4, s5, s6, s7, s8, s9, IMA]

config_1 = Kos.Setup()
Petzval = Kos.system(A, config_1)



W = 0.4861
sup = 1
AperVal = 50
AperType = "EPD"
Pup = Kos.PupilCalc(Petzval, sup, W, AperType, AperVal)

Pup.Samp = 15
Pup.Ptype = "hexapolar"
Pup.FieldType = "angle"
Pup.FieldX =1.0

X, Y, Z, P2V = Kos.Phase(Pup)

"""Zernike polynomial, number of terms"""
NC = 35
A = np.ones(NC)

Zcoef, Mat, RMS2Chief, RMS2Cent, F_ERR = Kos.Zernike_Fitting(X, Y, Z, A)

"""" Printing coefficients """
for i in range(0, NC):
    print("z", i + 1, "  ", "{0:.8f}".format(float(Zcoef[i])), ":", Mat[i])

print("(RMS) Fitting error: ", F_ERR)
print(RMS2Chief, "RMS(to chief) From fitted coefficents")
print(RMS2Cent, "RMS(to centroid) From fitted coefficents")



# !/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Examp Tel 2M Wavefront Fitting"""

import os
import sys
import matplotlib.pyplot as plt
import numpy as np
import pkg_resources

""" Looking for if KrakenOS is installed, if not, it assumes that
an folder downloaded from github is run"""

required = {'KrakenOS'}
installed = {pkg.key for pkg in pkg_resources.working_set}
missing = required - installed

if missing:
    print("Not installed")
    import sys
    sys.path.append("../..")


import KrakenOS as Kos

# ______________________________________#

currentDirectory = os.getcwd()
sys.path.insert(1, currentDirectory + '/library')

# ______________________________________#

P_Obj = Kos.surf()
P_Obj.Rc = 0
P_Obj.Thickness = 1000*0 + 3452.2
P_Obj.Glass = "AIR"
P_Obj.Diameter = 1059. * 2.0
P_Obj.Drawing=0

# ______________________________________#

Thickness = 3452.2
M1 = Kos.surf()
M1.Rc = -9638.0
M1.Thickness = -Thickness
M1.k = -1.07731
M1.Glass = "MIRROR"
M1.Diameter = 1.059E+003 * 2.0
M1.InDiameter = 250 * 2.0
M1.TiltY = 0.0
M1.TiltX = 0.0
M1.AxisMove = 0
# ______________________________________#

M2 = Kos.surf()
M2.Rc = -3930.0
M2.Thickness = Thickness + 1037.525880
M2.k = -4.3281
M2.Glass = "MIRROR"
M2.Diameter = 336.5 * 2.0
M2.TiltY = 0.0
M2.TiltX = 0.0
M2.DespY = 0.0
""" Se inclina el secundario """
M2.DespX = 1.0
M2.AxisMove = 0

# ______________________________________#


N1 = Kos.surf()
N1.Thickness = 720
N1.Glass = "NULL"
N1.Diameter = 100
N1.TiltX = 10
N1.Order = 0
N1.AxisMove = 1

############################

Colim = Kos.surf()
Colim.Rc = -1440.0
Colim.k = -1.0
Colim.Thickness = -720
Colim.Glass = "MIRROR"
Colim.Diameter = 400
Colim.TiltX = 0
Colim.Order = 0

############################
N2 = Kos.surf()
N2.Glass = "NULL"
N2.DespY =125.9836755
N2.AxisMove = 1
############################

N3 = Kos.surf()
N3.Glass = "NULL"
N3.TiltX =-71
N3.AxisMove = 1

#############################

G1 = Kos.surf()
G1.Glass = "MIRROR"
G1.Diameter = 300
G1.Grating_D = (1/0.079)
G1.Diff_Ord = -41
G1.Grating_Angle = 10.

#############################

N4 = Kos.surf()
N4.Glass = "NULL"
N4.TiltX = -N3.TiltX
N4.AxisMove = 1

#############################

N5 = Kos.surf()
N5.Glass = "NULL"
N5.Thickness = 1000
N5.TiltX = -N1.TiltX
N5.AxisMove = 1

#############################

# N6 = Kos.surf()
# N6.Glass = "NULL"
# N6.TiltX = -4.95736765
# N6.DespY = 47.70641809
# N6.AxisMove = 1

# #############################

# N7 = Kos.surf()
# N7.Glass = "NULL"
# N7.TiltZ = 90

# #############################

# N8 = Kos.surf()
# N8.Glass = "NULL"
# N8.TiltX = 20.7

# #############################

# G2 = Kos.surf()
# G2.Glass = "AIR"
# G2.Diameter = 18300
# G2.Thickness = 100
# # G2.Grating_D = 0#1/0.3
# # G2.Diff_Ord = 0
# G2.Grating_Angle = 0.0



P_Ima = Kos.surf()
P_Ima.Diameter = 1900
P_Ima.Rc = -1000.
P_Ima.Glass = "AIR"
P_Ima.Name = "Plano imagen"

# ______________________________________#

A = [P_Obj, M1, M2, N1, Colim, N2, N3, G1,N4, N5, P_Ima]
configuracion_1 = Kos.Setup()
Telescopio = Kos.system(A, configuracion_1)

# ______________________________________#
Telescopio = Kos.system(A, configuracion_1)
Rayos = Kos.raykeeper(Telescopio)

# ______________________________________#

tam = 9
rad = 2100 / 2
tsis = len(A) - 1

# ______________________________________#



# ______________________________________#

# for i in range(-tam, tam + 1):
for j in range(-tam, tam + 1):
    x_0 = (0 / tam) * rad
    y_0 = (j / tam) * rad
    r = np.sqrt((x_0 * x_0) + (y_0 * y_0))
    if r < rad:
        print(0)
        tet = 0.0
        pSource_0 = [x_0, y_0, 0.0]
        dCos = [0.0, np.sin(np.deg2rad(tet)), np.cos(np.deg2rad(tet))]
        W = .55
        Telescopio.Trace(pSource_0, dCos, W)
        Rayos.push()

# ______________________________________#


Kos.display2d(Telescopio, Rayos,0,0)

# Kos.display2d(Telescopio, Rayos,1,0)

print(Telescopio.EFFL)
X, Y, Z, L, M, N = Rayos.pick(-1)

# ______________________________________#

plt.plot(X, Y, 'x')
plt.xlabel('X')
plt.ylabel('Y')
plt.title('Spot Diagram')
plt.axis('square')
plt.show()

